// ----------------------------------------------------------------------------
//
//  Copyright (C) 2021 Arthur Benilov <arthur.benilov@gmail.com>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------

#pragma once

#include "aeolus/globals.h"
#include "aeolus/levelmeter.h"

namespace ui {

/**
 * @brief Level indicator.
 * 
 * This widget is coupled with a LevelMeter to display
 * the level as a horizontal or vertical bar.
 */
class LevelIndicator : public juce::Component,
                       public juce::Timer
{
public:

    enum class Orientation
    {
        Horizontal,
        Vertical
    };

    LevelIndicator() = delete;
    LevelIndicator(aeolus::LevelMeter& meter, Orientation orientation);

    void setSkew(float s) noexcept { _skew = s; }

    // juce::Component
    void paint(juce::Graphics& g) override;

    // juce::Timer
    void timerCallback() override;

private:

    void paintHorizontal(juce::Graphics& g);
    void paintVertical(juce::Graphics& g);

    juce::ColourGradient getColourGradient();

    aeolus::LevelMeter& _levelMeter;
    Orientation _orientation;
    float _skew;

    float _peakLevel;

    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(LevelIndicator)
};

} // namespace ui
